/*
 *  ServCtl.c
 *  
 *  Author	: Ahmed Obied (ahmed.obied@gmail.com)
 *	Version	: 0.1
 *
 *	DrvLoader is a graphical tool for loading and unloading Windows drivers
 *
 */
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "resource.h"

#define SERVCTL		"ServCtl"
#define LOAD		0x0
#define UNLOAD		0x1

HWND ghWnd;

VOID Error(PCHAR msg);

LPVOID MemAlloc(
	DWORD size
	)
{
	HANDLE heap;
	LPVOID mem;

	heap = GetProcessHeap();
	if (heap == NULL)
		Error("GetProcessHeap - can't get the handle to the heap");
	mem = HeapAlloc(heap,HEAP_ZERO_MEMORY,size);
	if (mem == NULL)
		Error("HeapAlloc - can't allocate memory on the heap");
	return mem;
}

VOID MemFree(
	LPVOID mem
	)
{
	HANDLE heap;

	heap = GetProcessHeap();
	if (heap == NULL)
		Error("GetProcessHeap - can't get the handle to the heap");
	HeapFree(heap,0,mem);
}


VOID Error(
	PCHAR msg
	)
{
	PCHAR errStr;

	errStr = (PCHAR)MemAlloc(MAX_PATH);
	sprintf_s(errStr,MAX_PATH,"An error has occurred.\n\n%s\n\nError Code: %d",msg,GetLastError());
	MessageBox(ghWnd,errStr,"ServCtl ERROR",MB_ICONERROR);
	MemFree(errStr);
}

PCHAR GetPath()
{
	PCHAR path;
	
	path = (PCHAR)MemAlloc(MAX_PATH);
	GetDlgItemText(ghWnd,IDC_PATH,path,MAX_PATH);
	if (strcmp(path,"") == 0) {
		MessageBox(ghWnd,"The path to image is empty",SERVCTL,MB_ICONINFORMATION);
		MemFree(path);	
		return NULL;
	}
	return path;
}

PCHAR GetName()
{
	PCHAR name;
	
	name = (PCHAR)MemAlloc(MAX_PATH);
	GetDlgItemText(ghWnd,IDC_NAME,name,MAX_PATH);
	if (strcmp(name,"") == 0) {
		MessageBox(ghWnd,"The name is empty",SERVCTL,MB_ICONINFORMATION);
		MemFree(name);	
		return NULL;
	}
	return name;
}

VOID ServiceOpt(
	PCHAR path, 
	PCHAR name, 
	DWORD opt
	)
{
	SC_HANDLE scManager, scHandle;

	scManager = OpenSCManager(NULL,NULL,SC_MANAGER_CREATE_SERVICE);
	if (scManager == NULL) {
		Error("OpenSCManager - can't open the service control manager");
		return;
	}
	if (opt == LOAD) {
		scHandle = CreateService(scManager,name,name,SERVICE_START|SERVICE_STOP|DELETE,
					SERVICE_KERNEL_DRIVER,SERVICE_DEMAND_START,SERVICE_ERROR_NORMAL,
					path,NULL,NULL,NULL,NULL,NULL);
		if (scHandle == NULL) {
			CloseServiceHandle(scManager);
			Error("CreateService - can't create the service");
			return;
		}
	}
	if (opt == LOAD) {
		scHandle = OpenService(scManager,name,SERVICE_START|DELETE);
		if (scHandle == NULL) {
			CloseServiceHandle(scManager);
			Error("OpenService - can't open service");
			return;
		}
		if (StartService(scHandle,NULL,NULL) == 0) {
			DeleteService(scHandle);
			CloseServiceHandle(scManager);
			CloseServiceHandle(scHandle);
			Error("StartService - can't start service");
			return;
		}
		MessageBox(ghWnd,"The service has been loaded successfully.",SERVCTL,MB_OK);
	}
	else
	if (opt == UNLOAD) {
		SERVICE_STATUS scStatus;

		scHandle = OpenService(scManager,name,SERVICE_STOP|DELETE);
		if (scHandle == NULL) {
			CloseServiceHandle(scManager);
			Error("OpenService - can't open service");
			return;
		}
		if (ControlService(scHandle,SERVICE_CONTROL_STOP,&scStatus) == 0) {
			CloseServiceHandle(scManager);
			CloseServiceHandle(scHandle);
			Error("ControlService - can't control service");
			return;
		}
		if (DeleteService(scHandle) == 0) {
			CloseServiceHandle(scManager);
			CloseServiceHandle(scHandle);
			Error("DeleteService - can't delete service");
			return;
		}
		MessageBox(ghWnd,"The service has been unloaded successfully.",SERVCTL,MB_OK);
	}
	CloseServiceHandle(scHandle);
	CloseServiceHandle(scManager);
}

INT_PTR CALLBACK DrvOptions(
	HWND hWnd, 
	UINT msg, 
	WPARAM wParam, 
	LPARAM lParam
	)
{
	PCHAR path, name;

	switch (msg) {
		case WM_INITDIALOG:
			ghWnd = hWnd;
			return TRUE;
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
				case IDC_LOAD:
					name = GetName();
					if (name != NULL) {
						path = GetPath();
						if (path != NULL) {
							ServiceOpt(path,name,LOAD);
							MemFree(name);
							MemFree(path);
						}
					}
					return TRUE;
				case IDC_UNLOAD:	
					name = GetName();
					if (name != NULL) {
						ServiceOpt(NULL,name,UNLOAD);
						MemFree(name);
					}
					return TRUE;
			}
			return TRUE;
		case WM_CLOSE:
			EndDialog(hWnd,LOWORD(wParam));
			return TRUE;
	}
	return FALSE;
}

int WINAPI WinMain(
	HINSTANCE hInstance, 
	HINSTANCE hPrevInstance, 
	LPSTR lpCmdLine, 
	int nCmdShow
	)
{
	DialogBox(hInstance,MAKEINTRESOURCE(IDD_SERVCTL),NULL,DrvOptions);	
	return 0;
}
